package net.xdclass.shopmanager.controller;

import com.google.ortools.Loader;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import net.xdclass.shopmanager.model.*;
import net.xdclass.shopmanager.service.SubService;
import net.xdclass.shopmanager.util.JsonData;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collector;
import java.util.stream.Collectors;

import static net.xdclass.shopmanager.tool.Tool.nvl;
import static net.xdclass.shopmanager.util.Knapsack.solve;

@RestController
@Slf4j(topic = "xt-special1")
@RequestMapping("/Sub")
@Api(tags = "余坯组板模块", value = "获取数据组板相关接口")
public class SubController {
    @Autowired
    private SubService subService;

    private final static Logger logger = LoggerFactory.getLogger(SubController.class);

    //region ***全局变量过程数据***
    private static List<GPreOrder> gltOrder = new ArrayList<>();

    //初次订单分组
    private static List<GGrpOrderFir> gltGrpOrderFir = new CopyOnWriteArrayList<>();

    //二次计算分组
    private static List<GCalGrp> gltGCalGrp = new CopyOnWriteArrayList<>();

    //计算结果数据
    private static List<GGrpCalRslt> gltGGrpCalRslt = new CopyOnWriteArrayList<>();

    //组板板坯表
    private static List<PmPdSlabDO> pmPdSlabDOList = new ArrayList<>();
    //大板粗切位置表
    private static List<PmPdPlateCutDO> pmPdPlateCutDOList = new ArrayList<>();
    //组合子板位置表
    private static List<PmPdBlockPosDO> pmPdBlockPosDOList = new ArrayList<>();
    //组板合同总量表
    private static List<PmPdContSumDO> pmPdContSumDOList = new ArrayList<>();
    //组板小板位置表
    private static List<PmSmallPltPosDO> pmSmallPltPosDOList = new ArrayList<>();
    //命令板坯表
    private static List<PmCmdSlabDO> pmCmdSlabDOList = new ArrayList<>();
    //命令大板表
    private static List<PmCmdLargePltDO> pmCmdLargePltDOList = new ArrayList<>();
    //命令参数表
    private static List<PmCmdContDO> pmCmdContDOList = new ArrayList<>();
    //命令小板表
    private static List<PmCmdSmallPltDO> pmCmdSmallPltDOList = new ArrayList<>();



    private static int intOrdNum = 0;

    private static int intGrpNum = 0;

    //endregion

    //region   ***全局变量获取基础数据***

    //板坯信息
    private static List<SlabInfoDO> plistSlab;
    //合同信息
    private static List<PmProContDO> plistOrder;
    //生产合同产品属性表
    private static List<PmProContProdAttrDO> plistPmProContProdAttrDO;
    //合同产线工艺参数表
    private static List<PmContMscTechDO> plistPmContMscTechDO;
    //组板设计规则集
    private static List<PmPdRuleSetDO> plistPmPdRuleSetDO;
    //组板设计规则集
    private static List<PmPdRuleItemDO> plistPmPdRuleItemDO;
    //最小轧制厚度公差范围基准表
    private static List<PmMinThickTolBtDO> plistPmMinThickTolBtDO;
    //钢板轧制规格对应收得率基准表
    private static List<PmPlateSpecYieldBtDO> plistPmPlateSpecYieldBtDO;
    //强度级别对照基准表
    private static List<PmStCodeBtDO> plistPmStCodeBtDO;
    //粗切位置决定基准表
    private static List<PmCutPlaceBtDO> plistPmCutPlaceBtDO;
    //组板常数基准表
    private static List<PmPdConstBtDO> plistPmPdConstBtDO;
    //钢板切缝切头尾范围基准表
    private static List<PmPlateCutLenBtDO> plistPmPlateCutLenBtDO;
    //组板临时收得率基准表
    private static List<PmPdTmpYieldBtDO> plistPmPdTmpYieldBtDO;
    //钢板切边量基准表
    private static List<PmPlateTrimBtDO> plistPmPlateTrimBtDO;
    //余材组板内部钢种以优充次对照表
    private static List<PmSlabReplBtDO> plistPmSlabReplBtDO;
    //热处理炉基准规格表
    private static List<PmHeatTreatFurnaceBtDO> plistPmHeatTreatFurnaceBtDO;
    //特殊钢种余量增加表
    private static List<PmSpeStAddvBtDO> plistPmSpeStAddvBtDO;
    //合同处理成分结果数据
    private static List<QdToLineupResultCheDO> plistQdToLineupResultCheDO;
    //合同处理坯料性能要求
    private static List<QdToLineupResultSlabphyDO> plistQdToLineupResultSlabphyDO;
    //炉次工序成分代表实绩表
    private static List<QeTqHeatcheStandResultDO> plistQeTqHeatcheStandResultDO;
    //炉次待委托表
    private static List<QeTqPonoWaitTestDO> plistQeTqPonoWaitTestDO;
    //坯料低倍委托实绩表
    private static List<QeTqSlabMacroResultDO> plistQeTqSlabMacroResultDO;
    //板坯信息表
    private static List<PmPdSlabInfoDO> plistPmPdSlabInfoDO;  //TODO 需要添加此表信息的查询

    //endregion

    @ApiOperation("求解处理")
    @PostMapping("Solver")
    public JsonData Solver() {
        // 测试数据
        long[] values = {360, 83, 59, 130, 431, 67, 230, 52, 93, 125, 670, 892, 600, 38, 48, 147,
                78, 256, 63, 17, 120, 164, 432, 35, 92, 110, 22, 42, 50, 323, 514, 28, 87, 73, 78, 15, 26,
                78, 210, 36, 85, 189, 274, 43, 33, 10, 19, 389, 276, 312};
        long[][] weights = {{7, 1, 30, 22, 80, 94, 11, 81, 70, 64, 59, 18, 1, 36, 3, 8, 15, 42, 9,
                1, 42, 47, 52, 32, 26, 48, 55, 6, 29, 84, 2, 4, 18, 56, 7, 29, 93, 44, 71, 3, 86, 66, 31,
                65, 1, 79, 20, 65, 52, 13}};
        long[] capacities = {7};
        //初始化方案
        Loader.loadNativeLibraries();
        // 调用 solve 方法来解决问题
        ArrayList<Long> ltInt = solve(values, weights, capacities);
        for (int i = 0; i < ltInt.size(); i++) {
            System.out.println(ltInt.get(i));
        }
        return JsonData.buildSuccess();
    }

    @ApiOperation("数据处理")
    @PostMapping("SubData")
    public JsonData SubData() {

        //获取所有数据
        GetDataInfo(null);

        //数据预处理
        SetPreData();

        logger.info("***初次分组数量***  " + gltGrpOrderFir.size());

        //循环板坯
        plistSlab.forEach(p -> {

                    try {
                        //二次分组
                        SetGrpDataBySlab(p);

                        logger.info("***二次分组数量***  " + gltGCalGrp.size());

                        if (gltGCalGrp.size() > 0) {

                            SetGroupCalRslt(p);
                            logger.info("结果数据板坯号：{}；数量：{}",p.getSlabNo(), gltGGrpCalRslt.size());
                        }
                        //if (gltGGrpCalRslt.size() > 0) {
                        //    SetSaveDate(p);
                        //}
                    } catch (Exception e) {
                        logger.info(e.toString());
                    }

                }
        );
        logger.info("*****处理结束*****");
        return JsonData.buildSuccess();
    }

    /*
     * 获取数据-多线程
     * 板坯数据
     * 订单数据
     * 配置信息
     * */
    public void GetDataInfo(Optional<SubParams> args) {

        logger.info("*****开始获取数据******");

        //region  ***多线程获取基础数据***

        // 创建线程池
        ExecutorService executorService = Executors.newFixedThreadPool(23);

        //初始化方案
        Loader.loadNativeLibraries();

        // 创建多个任务
        List<Callable<Void>> tasks = new ArrayList<>();

        //1板坯数据
        tasks.add(() -> {
            plistSlab = subService.getSlabInfo();
//            logger.info("******1板坯数据获取完毕*******");
            return null;
        });

        //2订单数据
        tasks.add(() -> {
            plistOrder = subService.getOrderInfo();
//            logger.info("******2订单数据获取完毕*******");
            return null;
        });

        //3生产合同产品属性表
        tasks.add(() -> {
            plistPmProContProdAttrDO = subService.getPmProCountProdAttr();
//            logger.info("******3生产合同产品属性表数据获取完毕*******");
            return null;
        });

        //4合同产线工艺参数表
        tasks.add(() -> {
            plistPmContMscTechDO = subService.getPmContMscTech();
//            logger.info("******4合同产线工艺参数表数据获取完毕*******");
            return null;
        });

        //5生产合同产品属性表
        tasks.add(() -> {
            // plistPmProContProdAttrDO = subService.getPmProCountProdAttr();
            Thread.sleep(10000); // 休眠10秒钟
//            logger.info("******5生产合同产品属性表数据获取完毕*******");
            return null;
        });

        //6组板设计规则集
        tasks.add(() -> {
            plistPmPdRuleSetDO = subService.getPmPdRuleSet();
//            logger.info("******6组板设计规则集数据获取完毕*******");
            return null;
        });

        //7组板设计规则集
        tasks.add(() -> {
            plistPmPdRuleItemDO = subService.getPmPdRuleItem();
//            logger.info("******7组板设计规则集数据获取完毕*******");
            return null;
        });

        //8最小轧制厚度公差范围基准表
        tasks.add(() -> {
            plistPmMinThickTolBtDO = subService.getPmMinThickTolBt();
//            logger.info("******8最小轧制厚度公差范围基准表数据获取完毕*******");
            return null;
        });

        //9钢板轧制规格对应收得率基准表
        tasks.add(() -> {
            plistPmPlateSpecYieldBtDO = subService.getPmPlateSpecYieldBt();
//            logger.info("******9钢板轧制规格对应收得率基准表数据获取完毕*******");
            return null;
        });

        //10强度级别对照基准表
        tasks.add(() -> {
            plistPmStCodeBtDO = subService.getPmStCodeBt();
//            logger.info("******10强度级别对照基准表数据获取完毕*******");
            return null;
        });

        //11粗切位置决定基准表
        tasks.add(() -> {
            plistPmCutPlaceBtDO = subService.getPmCutPlaceBt();
//            logger.info("******11粗切位置决定基准表数据获取完毕*******");
            return null;
        });

        //12组板常数基准表
        tasks.add(() -> {
            plistPmPdConstBtDO = subService.getPmPdConstBt();
//            logger.info("******12组板常数基准表数据获取完毕*******");
            return null;
        });

        //13钢板切缝切头尾范围基准表
        tasks.add(() -> {
            plistPmPlateCutLenBtDO = subService.getPmPlateCutLenBt();
//            logger.info("******13钢板切缝切头尾范围基准表数据获取完毕*******");
            return null;
        });

        //14组板临时收得率基准表
        tasks.add(() -> {
            plistPmPdTmpYieldBtDO = subService.getPmPdTmpYieldBt();
//            logger.info("******14组板临时收得率基准表数据获取完毕*******");
            return null;
        });

        //15钢板切边量基准表
        tasks.add(() -> {
            plistPmPlateTrimBtDO = subService.getPmPlateTrimBt();
//            logger.info("******15钢板切边量基准表数据获取完毕*******");
            return null;
        });

        //16余材组板内部钢种以优充次对照表
        tasks.add(() -> {
            plistPmSlabReplBtDO = subService.getPmSlabReplBt();
//            logger.info("******16余材组板内部钢种以优充次对照表数据获取完毕*******");
            return null;
        });

        //17热处理炉基准规格表
        tasks.add(() -> {
            plistPmHeatTreatFurnaceBtDO = subService.getPmHeatTreatFurnaceBt();
//            logger.info("******17热处理炉基准规格表数据获取完毕*******");
            return null;
        });

        //18特殊钢种余量增加表
        tasks.add(() -> {
            plistPmSpeStAddvBtDO = subService.getPmSpeStAddvBt();
//            logger.info("******18特殊钢种余量增加表数据获取完毕*******");
            return null;
        });

        //19合同处理成分结果数据
        tasks.add(() -> {
            plistQdToLineupResultCheDO = subService.getQdToLineupResultChe();
//            logger.info("******19合同处理成分结果数据获取完毕*******");
            return null;
        });

        //20合同处理坯料性能要求
        tasks.add(() -> {
            plistQdToLineupResultSlabphyDO = subService.getQdToLineupResultSlabphy();
//            logger.info("******20合同处理坯料性能要求数据获取完毕*******");
            return null;
        });

        //21炉次工序成分代表实绩表
        tasks.add(() -> {
            plistQeTqHeatcheStandResultDO = subService.getQeTqHeatcheStandResult();
//            logger.info("******21炉次工序成分代表实绩表数据获取完毕*******");
            return null;
        });

        //22炉次待委托表
        tasks.add(() -> {
            plistQeTqPonoWaitTestDO = subService.getQeTqPonoWaitTest();
//            logger.info("******22炉次待委托表数据获取完毕*******");
            return null;
        });

        //23坯料低倍委托实绩表
        tasks.add(() -> {
            plistQeTqSlabMacroResultDO = subService.getQeTqSlabMacroResult();
//            logger.info("******23坯料低倍委托实绩表数据获取完毕*******");
            return null;
        });

        // 执行所有任务
        try {
            executorService.invokeAll(tasks);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        logger.info("1:板坯数据：" + plistSlab.size());
        logger.info("2:订单数据：" + plistOrder.size());
        logger.info("3:生产合同产品属性表：" + plistPmProContProdAttrDO.size());
        logger.info("4:合同产线工艺参数表：" + plistPmContMscTechDO.size());
        logger.info("5:组板设计规则集：" + plistPmPdRuleSetDO.size());
        logger.info("6:组板设计规则集详细：" + plistPmPdRuleItemDO.size());
        logger.info("7:最小轧制厚度公差范围基准表：" + plistPmMinThickTolBtDO.size());
        logger.info("8:钢板轧制规格对应收得率基准表：" + plistPmPlateSpecYieldBtDO.size());
        logger.info("9:强度级别对照基准表：" + plistPmStCodeBtDO.size());
        logger.info("10:粗切位置决定基准表：" + plistPmCutPlaceBtDO.size());
        logger.info("11:组板常数基准表：" + plistPmPdConstBtDO.size());
        logger.info("12:钢板切缝切头尾范围基准表：" + plistPmPlateCutLenBtDO.size());
        logger.info("13:组板临时收得率基准表：" + plistPmPdTmpYieldBtDO.size());
        logger.info("14:钢板切边量基准表：" + plistPmPlateTrimBtDO.size());
        logger.info("15:余材组板内部钢种以优充次对照表：" + plistPmSlabReplBtDO.size());
        logger.info("16:热处理炉基准规格表：" + plistPmHeatTreatFurnaceBtDO.size());
        logger.info("17:特殊钢种余量增加表：" + plistPmSpeStAddvBtDO.size());
        logger.info("18:合同处理成分结果数据：" + plistQdToLineupResultCheDO.size());
        logger.info("19:合同处理坯料性能要求：" + plistQdToLineupResultSlabphyDO.size());
        logger.info("20:炉次工序成分代表实绩表：" + plistQeTqHeatcheStandResultDO.size());
        logger.info("21:炉次待委托表：" + plistQeTqPonoWaitTestDO.size());
        logger.info("22:坯料低倍委托实绩表：" + plistQeTqSlabMacroResultDO.size());

        // 关闭线程池
        executorService.shutdown();

        //endregion  多线程获取基础数据

    }

    /*
     * 数据预处理
     * */
    public void SetPreData() {
        //订单数据处理
        SetOrderLackData();

        //订单根据尺寸初次分组
        SetGroupOrderFir();

    }

    /*
     * 二次分组根据板坯性能删选分组
     * */
    public void SetGrpDataBySlab(SlabInfoDO SInfo) {

//        gltGrpOrderFir.parallelStream().forEach(e -> {
//
//            // gltGCalGrp 产生可计算分组
//
//            //压缩比  Compression ratio
//
//            //宽展比 Wide ratio
//
//            //钢种标准对比
//
//            GCalGrp GCG = new GCalGrp();
//
//            e.getLtInfoGPreOrder().parallelStream().forEach(f -> {
//
//                //化学成分  chemical composition
//                if (!ChkChemical(SInfo, f)) {
//                    return;
//                }
//                //低倍检测  low-power check
//                if (!ChkLowPower(SInfo, f)) {
//                    return;
//                }
//
//                GCG.setLtInfoGPreOrder(new ArrayList<>());
//                GCG.getLtInfoGPreOrder().add(f);//添加订单信息
//            });
//
//            //存在订单，添加可计算分组
//            if (GCG.getLtInfoGPreOrder().size() > 0) {
//                gltGCalGrp.add(GCG);
//            }
//
//        });

        gltGrpOrderFir.parallelStream().forEach(e -> {

            // gltGCalGrp 产生可计算分组

            //压缩比  Compression ratio

            //宽展比 Wide ratio

            //钢种标准对比

            GCalGrp GCG = new GCalGrp();
            GCG.setLtInfoGPreOrder(new ArrayList<>());
            e.getLtInfoGPreOrder().forEach(f -> {

                //化学成分  chemical composition
                if (!ChkChemical(SInfo, f)) {
                    return;
                }
                //低倍检测  low-power check
                if (!ChkLowPower(SInfo, f)) {
                    return;
                }

                //GCG.setLtInfoGPreOrder(new ArrayList<>());
                GCG.getLtInfoGPreOrder().add(f);//添加订单信息
            });

            //存在订单，添加可计算分组
            if (GCG.getLtInfoGPreOrder().size() > 0) {

                //GCG.getLtInfoGPreOrder().stream().filter(e->e.)
                //最大宽度
                OptionalInt maxWidth = GCG.getLtInfoGPreOrder().stream()
                        .mapToInt(GPreOrder::getWid)
                        .max();
                //最大宽度余量
                OptionalInt maxAddWidth = GCG.getLtInfoGPreOrder().stream()
                        .mapToInt(GPreOrder::getWidthAddv)
                        .max();

                //宽展比   保留2位小数
                BigDecimal decWidRate = BigDecimal.valueOf(maxWidth.orElse(0)).divide(SInfo.getSlabActWid(), 2, RoundingMode.HALF_UP);

                //最大厚度公差最小值
                Optional<BigDecimal> maxTolThkMin = GCG.getLtInfoGPreOrder().stream()
                        .map(GPreOrder::getThickTolMin)
                        .reduce(BigDecimal::max);

                //轧制厚度   厚度+厚度公差下限最大值
                BigDecimal decThk = e.getThk().add(maxTolThkMin.orElse(BigDecimal.ZERO));

                //轧制宽度   最大宽度+宽度余量
                int intWid = maxWidth.orElse(0) + maxAddWidth.orElse(0);

                //切边
                int intTrimWid = 0;
                //切边量
                if (plistPmPlateTrimBtDO.stream().anyMatch(p -> p.getMscLineNo().equals("1030")
                        && p.getPlateThickFrom().compareTo(decThk) <= 0
                        && p.getPlateThickTo().compareTo(decThk) >= 0
                        && p.getCombinePlateWidthFrom().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) <= 0
                        && p.getCombinePlateWidthTo().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) >= 0
                        && p.getWidenFrom().compareTo(decWidRate) <= 0
                        && p.getWidenTo().compareTo(decWidRate) >= 0
                )) {

                    intTrimWid = plistPmPlateTrimBtDO.stream().filter(p -> p.getMscLineNo().equals("1030")
                            && p.getPlateThickFrom().compareTo(decThk) <= 0
                            && p.getPlateThickTo().compareTo(decThk) >= 0
                            && p.getCombinePlateWidthFrom().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) <= 0
                            && p.getCombinePlateWidthTo().compareTo(BigDecimal.valueOf(maxWidth.orElse(0) + maxAddWidth.orElse(0))) >= 0
                            && p.getWidenFrom().compareTo(decWidRate) <= 0
                            && p.getWidenTo().compareTo(decWidRate) >= 0
                    ).collect(Collectors.toList()).get(0).getTrimWidth().intValue();
                }
                //轧制宽度= 最大宽度 + 最大宽度余量 + 切边量
                intWid = intWid + intTrimWid;
                //宽度
                GCG.setWid(intWid);

                //厚度
                GCG.setThk(decThk);

                //切边
                GCG.setWid_Add(intTrimWid);

                gltGCalGrp.add(GCG);
            }

        });

    }

    //region  验证方法

    /*
     * 化学成分判断
     * */
    public boolean ChkChemical(SlabInfoDO SInfo, GPreOrder GrpOrd) {

        //        - 炉次工序成分代表实绩表QE_TQ_HEATCHE_STAND_RESULT      =>  制造命令号PONO
        //        - 合同处理成分结果数据表QD_TO_LINEUP_RESULT_CHE  =>合同号+成分类型CHE_TYPE=RX

        //炉次化学成分

        if (!plistQeTqHeatcheStandResultDO.stream().anyMatch(p -> p.getPono().equals(SInfo.getPono()))) {
            //ltQthsr=plistQeTqHeatcheStandResultDO.stream().filter(p->p.getPono().equals(SInfo.getPono())).collect(Collectors.toList());
            return false;
        }

        //订单要求化学成分
//        logger.info("化学成分订单号：" + GrpOrd.getOrderNo());
        List<QdToLineupResultCheDO> ltLrc = new ArrayList<>();
//        int i =plistQdToLineupResultCheDO.stream().filter(p->p.getCheType().equals(null)).collect(Collectors.toList()).size();
//        logger.info("化学成分订单号：{}-{}" , GrpOrd.getOrderNo(),i);
        if (plistQdToLineupResultCheDO.stream().anyMatch(p -> p.getOrderNo().equals(GrpOrd.getOrderNo()) && p.getCheType().equals("RX"))) {
            ltLrc = plistQdToLineupResultCheDO.stream().filter(p -> p.getOrderNo().equals(SInfo.getOrderNo()) && p.getCheType().equals("RX")).collect(Collectors.toList());
        }

        //信息不存在返回
        if (ltLrc.size() == 0) {
            return false;
        }

        List<QeTqHeatcheStandResultDO> ltQthsr = plistQeTqHeatcheStandResultDO.stream().filter(p -> p.getPono().equals(SInfo.getPono())).collect(Collectors.toList());

        //判断合同的每个化学成分是否满足条件
        boolean blnChk = ltLrc.stream().anyMatch(p -> {
            if (!ltQthsr.stream().anyMatch(e -> e.getElm().equals(p.getElm())
                    && (e.getTestOriginalResult().compareTo(NumberUtils.createBigDecimal(p.getElmMin())) >= 0
                    && e.getTestOriginalResult().compareTo(NumberUtils.createBigDecimal(p.getElmMax())) <= 0))) {
                return false;
            }
            return true;
        });

        if (!blnChk) {
            return false;
        }
        return true;
    }

    /*
     * 低倍检测
     * */
    public boolean ChkLowPower(SlabInfoDO SInfo, GPreOrder GrpOrd) {

        // - 委托  QE_TQ_PONO_WAIT_TEST => 制造命令号PONO
        // - 坯料低倍委托实绩表QE_TQ_SLAB_MACRO_RESULT => 制造命令号PONO
        // - 合同处理坯料性能要求表QD_TO_LINEUP_RESULT_SLABPHY = > 合同号

        //  1、先通过制造命令号PONO查询QE_TQ_PONO_WAIT_TEST是否存在委托原因(该字段为多选字段内容)TEST_REASON=1的数据
        //    ，如未查询出数据则RESULT=1，如存在则通过制造命令号PONO抓取坯料低倍委托实绩表QE_TQ_SLAB_MACRO_RESULT
        //    ，获取实绩低倍数据，遍历合同号，再通过合同号查询合同处理坯料性能要求表QD_TO_LINEUP_RESULT_SLABPHY，获取该合同下低倍数据。
        //  2、对查出的数据进行对比，例如：合同中心偏析A字段等于1表示有要求，如中心偏析A实绩无数据则合同校验结果为不合格
        //    ，如对应实绩有数据则判断合同中心偏析A上限字段是否大于中心偏析A实绩，如大于则这个合同中心偏析A子项目合格
        //    ，如小于则合同校验结果为不合格，如合同中心偏析A字段等于0表示无要求，则不进行判断，依次判断到等轴晶率
        //    ，判断到合同其他缺陷A时需要特殊处理，如其他缺陷A等于1，之后拿合同其他缺陷A类型字段与实绩的其他缺陷A、B、C、D、E进行对比
        //    ，判断是否有相同的值，如无则合同校验结果为不合格，有则拿对应的其他缺陷A、B、C、D、E实绩与合同其他缺陷A上限进行比较
        //    ，如合同其他缺陷A上限大于其他缺陷A、B、C、D、E实绩，则这个合同其他缺陷A子项目合格，如小于则合同校验结果为不合格
        //    ，依次判断合同其他缺陷A、B、C、D、E，判定到合同校验结果为不合格时可以终止此层的循环，记录对应数据
        //    ，进入下一个合同号的循环，合格的数据也需要记录。

        //低倍合同要求表
        List<QdToLineupResultSlabphyDO> ltQthsr = plistQdToLineupResultSlabphyDO.stream().filter(p -> p.getOrderNo().equals(GrpOrd.getOrderNo())).collect(Collectors.toList());

        //判断合同是否需要判断低倍
        if (ltQthsr.size() == 0) {
            return true;
        }

        //委托  是否存在委托
        if (!plistQeTqPonoWaitTestDO.stream().anyMatch(p -> p.getPono().equals(SInfo.getPono()) && p.getTestReason().equals("1"))) {
            //ltQthsr=plistQeTqHeatcheStandResultDO.stream().filter(p->p.getPono().equals(SInfo.getPono())).collect(Collectors.toList());
            return false;
        }

        //坯料低倍委托实绩表
        List<QeTqSlabMacroResultDO> ltSmr = new ArrayList<>();
        if (plistQeTqSlabMacroResultDO.stream().anyMatch(p -> p.getPono().equals(SInfo.getPono()))) {
            ltSmr = plistQeTqSlabMacroResultDO.stream().filter(p -> p.getPono().equals(SInfo.getPono())).collect(Collectors.toList());
        }

        //信息不存在返回
        if (ltSmr.size() == 0) {
            return false;
        }
        //实例一个新的结果数据
        List<QeTqSlabMacroResultDO> ltSmrReslt = ltSmr;

        //中心偏析A
        if (ltQthsr.stream().anyMatch(p -> p.getCenterSgrgA().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCenterSgrgAResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCenterSgrgAMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCenterSgrgAResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }

        //        中心偏析B
        if (ltQthsr.stream().anyMatch(p -> p.getCenterSgrgB().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCenterSgrgBResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCenterSgrgBMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCenterSgrgBResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        中心偏析C
        if (ltQthsr.stream().anyMatch(p -> p.getCenterSgrgC().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCenterSgrgCResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCenterSgrgCMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCenterSgrgCResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        中心疏松
        if (ltQthsr.stream().anyMatch(p -> p.getCenterPorosity().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCenterPorosityResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCenterPorosityMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCenterPorosityResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        白亮带
        if (ltQthsr.stream().anyMatch(p -> p.getWhiteBand().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getWhiteBandResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getWhiteBandMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getWhiteBandResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        中间裂纹
        if (ltQthsr.stream().anyMatch(p -> p.getCrackCenter().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCrackCenterResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCrackCenterMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCrackCenterResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        三角区裂纹级别
        if (ltQthsr.stream().anyMatch(p -> p.getTriCrackGrade().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getTriCrackGradeResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getTriCrackGradeMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getTriCrackGradeResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        角裂级别
        if (ltQthsr.stream().anyMatch(p -> p.getAngleCrackGrade().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getAngleCrackGradeResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getAngleCrackGradeMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getAngleCrackGradeResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        三氧化二铝夹杂
        if (ltQthsr.stream().anyMatch(p -> p.getInclu().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getIncluResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getIncluMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getIncluResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        缩孔
        if (ltQthsr.stream().anyMatch(p -> p.getHore().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getHoreResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getHoreMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getHoreResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        蜂窝状气泡
        if (ltQthsr.stream().anyMatch(p -> p.getCellularBubble().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getCellularBubbleResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getCellularBubbleMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getCellularBubbleResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        等轴晶率
        if (ltQthsr.stream().anyMatch(p -> p.getWafer().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getWaferResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getWaferMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getWaferResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        其他缺陷A
        if (ltQthsr.stream().anyMatch(p -> p.getOtherDefectA().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getOtherDefectAResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getOtherDefectAMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getOtherDefectAResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        其他缺陷B
        if (ltQthsr.stream().anyMatch(p -> p.getOtherDefectB().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getOtherDefectBResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getOtherDefectBMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getOtherDefectBResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        其他缺陷C
        if (ltQthsr.stream().anyMatch(p -> p.getOtherDefectC().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getOtherDefectCResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getOtherDefectCMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getOtherDefectCResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        其他缺陷D
        if (ltQthsr.stream().anyMatch(p -> p.getOtherDefectD().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getOtherDefectDResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getOtherDefectDMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getOtherDefectDResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }
        //        其他缺陷E
        if (ltQthsr.stream().anyMatch(p -> p.getOtherDefectE().equals("1"))) {
            if (ltSmrReslt.stream().anyMatch(p -> {
                String result = p.getOtherDefectEResult();
                return result != null && !result.isEmpty();
            })) {
                if (!ltQthsr.stream().anyMatch(p -> {
                    String strReqMax = p.getOtherDefectEMax();
                    String strRslt = ltSmrReslt.stream().findFirst().map(e -> e.getOtherDefectEResult()).orElse("");
                    int num1 = Integer.parseInt(strReqMax);
                    int num2 = Integer.parseInt(strRslt);
                    //结果大于上限
                    if (num2 > num1) {
                        return false;
                    }
                    return true;
                })) {
                    return false;
                }
            }
        }

        return true;
    }

    /*
     * 压缩比判断
     * */
    public boolean ChkCompress(SlabInfoDO SInfo, GGrpOrderFir GrpOrd) {

        return true;
    }

    /*
     * 宽展比判断
     * */
    public boolean ChkWid(SlabInfoDO SInfo, GGrpOrderFir GrpOrd) {

        return true;
    }

    //endregion

    // 同步方法，用于更新gltOrder数组中的元素
    public static synchronized void SetAllOrder(PmProContDO prPmProContDO, PmProContProdAttrDO ppcpa, PmContMscTechDO pcmt) {

        BigDecimal lackNum = prPmProContDO.getPdLackQty();
        if (lackNum.intValue() > 0) {
            for (int i = 0; i < lackNum.intValue(); i++) {
                GPreOrder GPO = new GPreOrder();
                GPO.setIDValue(intOrdNum++);
                GPO.setThk(ppcpa.getOrderThick());//厚度
                GPO.setWid(ppcpa.getOrderWidth().intValue());//宽度
                GPO.setWidthAddv(ppcpa.getWidthAddv().intValue());//宽度余量
                GPO.setLen(ppcpa.getOrderLen());//长度
                GPO.setThickTolMin(ppcpa.getThickTolMin());//厚度公差下限
                GPO.setOrderNo(prPmProContDO.getOrderNo());
                GPO.setInfoPmProContDo(prPmProContDO);//订单信息
                GPO.setInfoPmProContProdAttrDO(ppcpa);//订单要求信息
                GPO.setInfoPmContMscTechDO(pcmt);//订单产线信息

                gltOrder.add(GPO);
            }

        }

    }

    /*
     * 订单数据欠量处理
     * */
    public void SetOrderLackData() {

        //TODO:箭头函数订单多线程处理数据，升序，降序，重组。
        //使用 Stream.parallel() 来并行执行 for 循环  OrderNo  升序   PdLackQty 降序
        plistOrder.parallelStream()
                .sorted(Comparator.comparing(PmProContDO::getOrderNo)
                        .thenComparing(Comparator.comparing(PmProContDO::getPdLackQty).reversed()))
                .forEach(i -> {
                    PmProContProdAttrDO ppcpa = new PmProContProdAttrDO();
                    PmContMscTechDO pcmt = new PmContMscTechDO();

                    try {
                        if (plistPmProContProdAttrDO.stream().filter(p -> p.getOrderNo().equals(i.getOrderNo())).count() > 0) {
                            ppcpa = plistPmProContProdAttrDO.stream().filter(p -> p.getOrderNo().equals(i.getOrderNo())).findFirst().orElseThrow(() -> new NoSuchElementException("No PmProContProdAttrDO found with name Charlie " + i.getOrderNo()));

                            if (plistPmContMscTechDO.stream().filter(p -> p.getOrderNo().equals(i.getOrderNo()) && p.getMscLineNo().equals("1030")).count() > 0) {
                                // 可能抛出异常的代码
                                pcmt = plistPmContMscTechDO.stream().filter(p -> p.getOrderNo().equals(i.getOrderNo())).findFirst().orElseThrow(() -> new NoSuchElementException("No PmContMscTechDO found with name Charlie  " + i.getOrderNo()));
                                SetAllOrder(i, ppcpa, pcmt);
                            } else {
//                                logger.info("缺少产线信息：" + i.getOrderNo());
                            }
                        } else {
//                            logger.info("缺少属性信息：" + i.getOrderNo());
                        }

                    } catch (NullPointerException ex1) {
                        System.out.println("错误信息：" + ex1.getMessage());
                    }
                });

        //        //排序后产生新的List
        //      List<PmProContDO>  ltNewProContDO =   plistOrder.parallelStream()
        //                .sorted(Comparator.comparing(PmProContDO ::getOrderNo)
        //                        .thenComparing(Comparator.comparing(PmProContDO::getPdLackQty).reversed()))
        //                .collect(Collectors.toList());

        System.out.println("订单欠量数：" + gltOrder.size());
    }

    /*
     * 订单根据尺寸钢种初次分组
     * */
    public void SetGroupOrderFir() {

        //region 多线程  数据流处理订单分组

        gltOrder.parallelStream().forEach(p -> {

            if (gltGrpOrderFir.stream().anyMatch(e -> e.getWid().equals(p.getInfoPmProContProdAttrDO().getOrderWidth()) &&
                    e.getThk().equals(p.getInfoPmProContProdAttrDO().getOrderThick()) &&
                    e.getSG_STD().equals(p.getInfoPmProContProdAttrDO().getSgStd()))) {

                GGrpOrderFir GGOF = gltGrpOrderFir.stream().filter(e -> e.getWid().equals(p.getInfoPmProContProdAttrDO().getOrderWidth())
                        && e.getThk().equals(p.getInfoPmProContProdAttrDO().getOrderThick())
                        && e.getSG_STD().equals(p.getInfoPmProContProdAttrDO().getSgStd())
                ).findFirst().orElse(null);
                if (GGOF.getLtInfoGPreOrder() == null) {
                    GGOF.setLtInfoGPreOrder(new ArrayList<>());  // Ensure the list is initialized
                }
                GGOF.getLtInfoGPreOrder().add(p);

            } else {

                GGrpOrderFir GGOF = new GGrpOrderFir();
                GGOF.setIDValue(intGrpNum++);
                GGOF.setWid(p.getInfoPmProContProdAttrDO().getOrderWidth());
                GGOF.setThk(p.getInfoPmProContProdAttrDO().getOrderThick());
                //GGOF.setLen(p.getInfoPmProContProdAttrDO().getOrderLen());
                GGOF.setSG_STD(p.getInfoPmProContProdAttrDO().getSgStd());

                GGOF.setLtInfoGPreOrder(new ArrayList<>());  // Initialize the list
                GGOF.getLtInfoGPreOrder().add(p);
                gltGrpOrderFir.add(GGOF);
            }

        });

        //endregion

        //region 单线程  调试用
        //        if (gltOrder.size() > 0) {
        //            for (GPreOrder p : gltOrder) {
        //                boolean found = false;
        //
        //                // Iterate over gltGrpOrderFir to find a matching GGrpOrderFir
        //                for (GGrpOrderFir e : gltGrpOrderFir) {
        //                    if (e.getWid().equals(p.getInfoPmProContProdAttrDO().getOrderWidth()) &&
        //                            e.getThk().equals(p.getInfoPmProContProdAttrDO().getOrderThick()) &&
        //                            e.getLen().equals(p.getInfoPmProContProdAttrDO().getOrderLen()) &&
        //                            e.getSG_STD().equals(p.getInfoPmProContProdAttrDO().getSgStd())) {
        //                        if (e.getLtInfoGPreOrder() == null) {
        //                            e.setLtInfoGPreOrder(new ArrayList<>());  // Ensure the list is initialized
        //                        }
        //                        // If a match is found, add the order to LtInfoGPreOrder
        //                        e.getLtInfoGPreOrder().add(p);
        //                        found = true;
        //                        break;  // Exit loop once a match is found
        //                    }
        //                }
        //
        //                // If no match was found, create a new GGrpOrderFir and add the order
        //                if (!found) {
        //                    GGrpOrderFir GGOF = new GGrpOrderFir();
        //                    GGOF.setIDValue(intGrpNum++);
        //                    GGOF.setWid(p.getInfoPmProContProdAttrDO().getOrderWidth());
        //                    GGOF.setThk(p.getInfoPmProContProdAttrDO().getOrderThick());
        //                    GGOF.setLen(p.getInfoPmProContProdAttrDO().getOrderLen());
        //                    GGOF.setSG_STD(p.getInfoPmProContProdAttrDO().getSgStd());
        //
        //                    GGOF.setLtInfoGPreOrder(new ArrayList<>());  // Initialize the list
        //                    GGOF.getLtInfoGPreOrder().add(p);
        //                    gltGrpOrderFir.add(GGOF);
        //                }
        //            }
        //        }
        //endregion

        System.out.println("初始分组数量：" + gltGrpOrderFir.size());

    }


    public void SetGroupCalRslt(SlabInfoDO SInfo) {

        //轧机最大长度
        int intRollMaxLen = plistPmPdConstBtDO.stream().filter(p -> p.getRollLineNo().equals("30")).findFirst().orElse(null).getHpmMaxPLen().intValue();
        //轧机最小长度
        int intRollMinLen = plistPmPdConstBtDO.stream().filter(p -> p.getRollLineNo().equals("30")).findFirst().orElse(null).getHpmMinPLen().intValue();

        //gltGCalGrp.parallelStream().forEach(e->{
        gltGCalGrp.parallelStream().forEach(e -> {
            //实例化计算结果数组
            GGrpCalRslt GGCR = new GGrpCalRslt();
            //可轧制最大长度
            int intMaxLen = 0;
            //轧制最大宽度
            int intWid = e.getWid();
            //轧制最大厚度
            BigDecimal intThk = e.getThk();
            //板坯长度
            int intSlabLen = SInfo.getSlabActLen().intValue();

            //判断可轧制最大长度 TODO：厚度计算Decimal取整计算，会有误差，后续判断误差值修正计算
            if ((SInfo.getSlabActLen().multiply(SInfo.getSlabActThk()).multiply(SInfo.getSlabActWid()).divide(intThk.multiply(BigDecimal.valueOf(intWid)), RoundingMode.HALF_UP)).compareTo(BigDecimal.valueOf(intRollMaxLen)) > 0) {
                intMaxLen = intRollMaxLen;
            } else {
                intMaxLen = (SInfo.getSlabActLen().multiply(SInfo.getSlabActThk()).multiply(SInfo.getSlabActWid()).divide(intThk.multiply(BigDecimal.valueOf(intWid)), RoundingMode.HALF_UP)).intValue();
            }
            //不能小于可轧制的最小长度
            if (intMaxLen < intRollMinLen) {
                return;
            }

            GGCR.setCal_Wid(intWid);
            GGCR.setCal_Thk(intThk);
            GGCR.setCal_Len(intMaxLen);
            GGCR.setLtInfoGPreOrder(e.getLtInfoGPreOrder());

            gltGGrpCalRslt.add(GGCR);
        });

        //分组计算
//        gltGGrpCalRslt.parallelStream().forEach(p -> {
        gltGGrpCalRslt.parallelStream().forEach(p -> {
            // 测试数据
            //处理数据多处理
            List<Integer> ltValues= new ArrayList<>();
            List<Integer> ltweights= new ArrayList<>();
            List<String> ltOrder= new ArrayList<>();

            logger.info( "订单总数量：" + p.getLtInfoGPreOrder().size());

            //计算可处理数据最大值
            p.getLtInfoGPreOrder().forEach(e->{

                int intOrdCount = p.getCal_Len()/e.getLen();

                if(ltOrder.stream().filter(f->f.equals(e.getOrderNo())).count()==intOrdCount)
                {
                    return;
                }
                ltOrder.add(e.getOrderNo());//添加订单号
                ltValues.add(e.getIDValue());//价值
                ltweights.add(e.getLen());//长度
            });

            // 生成 values 和 weights 数组
            long[] values = new long[ltValues.size()];
            long[][] weights = new long[1][ltweights.size()];

            for (int i = 0; i < ltValues.size(); i++) {
                values[i] =ltValues.get(i);       // 从订单中获取 value
                weights[0][i] = ltweights.get(i)+5;   // 从订单中获取 weights
            }

            //是否展宽
            //String brnNeed="";
            //if (p.getCal_Wid()>SInfo.getSlabActWid().intValue()){
            //    brnNeed="1";//展宽
            //}else {
            //    brnNeed="0";//不展宽
            //}

            //TODO：轧制方向，轧制模式 没有其他值暂不考虑，之后数据完善后需要考虑
            PmPlateCutLenBtDO PPCLB = plistPmPlateCutLenBtDO.stream().
                    filter(f -> f.getMscLineNo().equals("1030")
                            && f.getBrrNeed().equals((p.getCal_Wid() > SInfo.getSlabActWid().intValue()) ? "1" : "0")
                            && f.getPlateThickFrom().compareTo(p.getCal_Thk()) <= 0
                            && f.getPlateThickTo().compareTo(p.getCal_Thk()) >= 0
                            && f.getSlabThickFrom().compareTo(SInfo.getSlabActThk()) <= 0
                            && f.getSlabThickTo().compareTo(SInfo.getSlabActThk()) >= 0
                            && f.getCombinePlateWidthFrom().compareTo(BigDecimal.valueOf(p.getCal_Wid())) <= 0
                            && f.getCombinePlateWidthTo().compareTo(BigDecimal.valueOf(p.getCal_Wid())) >= 0
                    ).findFirst().orElse(null);
            if (PPCLB == null) {
                return;
            }
            //切头
            int intHead = PPCLB.getHeadCut().intValue();
            //切尾
            int intTail = PPCLB.getTailCut().intValue();
            //去掉切头尾长度
            int intCap = p.getCal_Len() - intHead - intTail;

            long[] capacities = {intCap};

            //初始化方案
            Loader.loadNativeLibraries();

//            logger.info("values  Size:{}", p.getLtInfoGPreOrder().size());
//            for (int i = 0; i < ltweights.size(); i++) {
//
//                logger.info("weights:{}", ltweights.get(i));
//
//            }

            // 调用 solve 方法来解决问题
            ArrayList<Long> ltInt = solve(values, weights, capacities);

            //实例化实际订单号
            p.setLtActInfoGPreOrder(new ArrayList<>());
            //循环返回的订单插入到实际数组中
            for (int i = 0; i < ltInt.size(); i++) {
                int indexi = i;
                p.getLtActInfoGPreOrder().add(p.getLtInfoGPreOrder().stream().filter(m -> m.getIDValue() == ltInt.get(indexi)).collect(Collectors.toList()).get(0));
            }

            //根据实际订单再计算
            p.setHead_Len(intHead);
            p.setTail_Len(intTail);

        });

    }

    //设置保存数据
    public void SetSaveDate(SlabInfoDO SInfo){
        SetPmPdSlab(SInfo);//组板板坯表
        SetPmPdPlateCut();//大板粗切位置表
        SetPmPdBlockPos();//组合子板位置表
        SetPmPdContSum();//组板合同总量表
        SetPmSmallPltPos();//组板小板位置表
        SetPmCmdSlab();//命令板坯表
        SetPmCmdLargePlt();//命令大板表
        SetPmCmdCont();//命令参数表
        SetPmCmdSmallPlt();//命令小板表
    }

    //组板板坯表
    public void SetPmPdSlab(SlabInfoDO SInfo){
        for(int i = 0; i < gltGGrpCalRslt.size(); i++){
            GGrpCalRslt ggcr=gltGGrpCalRslt.get(i);
            PmPdSlabDO pmPdSlabDO = new PmPdSlabDO();
            pmPdSlabDO.setPono(SInfo.getPono());
            pmPdSlabDO.setSlabDivNo(BigDecimal.ZERO); //TODO 板坯分割号 未赋值  查询板坯号等于slab_no的组板板坯表的记录数+1
            pmPdSlabDO.setVirtualSlabNo(ggcr.getVirtualSlabNo());
            pmPdSlabDO.setSlabNo(ggcr.getSlabNo());
            pmPdSlabDO.setMatDesignKind("S");
            pmPdSlabDO.setPdStatusHp("12");
            pmPdSlabDO.setPdResp("781"); // TODO 组板责任者 未赋值  操作人id
            pmPdSlabDO.setFormPlateMode("FormPlateMode"); //TODO 组板方式 未赋值
            pmPdSlabDO.setPickModeLenDir(BigDecimal.ZERO); //TODO 取板方式_长度方向 未赋值
            pmPdSlabDO.setPickModeWidDir(BigDecimal.ZERO); //TODO 取板方式_宽度方向 未赋值
            pmPdSlabDO.setFurType("FurType"); //TODO 加热炉类型 未赋值
            pmPdSlabDO.setAutoManualFlag("M");
            pmPdSlabDO.setOrderRemainDiv("1");
            pmPdSlabDO.setCanDelFlag("1");
            List<String> orderNoList = ggcr.getLtActInfoGPreOrder().stream().map(GPreOrder::getOrderNo).distinct().collect(Collectors.toList());//根据实际合同信息 的合同号去重 取出板内合同个数
            pmPdSlabDO.setOrderNumInPlate(BigDecimal.valueOf(orderNoList.size()));
            pmPdSlabDO.setTotSmallPlateNum(BigDecimal.valueOf(ggcr.getLtActInfoGPreOrder().size()));

            Optional<PmContMscTechDO> resultPmContMscTechDO = plistPmContMscTechDO.stream()
                    .filter(item -> ggcr.getLtActInfoGPreOrder().get(0).getOrderNo().equals(item.getOrderNo())) //TODO 根据轴合同号查询合同工艺表  暂时取第一条合同
                    .findFirst();
            pmPdSlabDO.setAdTypeCode(resultPmContMscTechDO.get().getAdTypeCode());
            pmPdSlabDO.setAdModeCode(resultPmContMscTechDO.get().getAdModeCode());
            pmPdSlabDO.setCtrlRollCode(resultPmContMscTechDO.get().getCtrlRollCode());
            pmPdSlabDO.setCutModeCode(resultPmContMscTechDO.get().getCutModeCode());

            //TODO 缺少质量模块MES合同主档QD_TO_PO_ORDER
            pmPdSlabDO.setUltCode(resultPmContMscTechDO.get().getUltCode());//TODO 遍历组板合同，根据合同号从质量模块MES合同主档QD_TO_PO_ORDER读取ERP_UT_SPEC1探伤标准1、ERP_UT_SPEC2超声探伤标2、ERP_UT_SPEC3超声探伤标3，去重后所有数据拼接成一条记录

            String ultDetectMethodCode = "";
            String coolCode = "";
            // 遍历 组板合同列表
            for (GPreOrder gPreOrder : ggcr.getLtActInfoGPreOrder()) {
                String orderNo = gPreOrder.getOrderNo();
                String mscLineNo = gPreOrder.getInfoPmContMscTechDO().getMscLineNo();

                // 根据 orderNo 和 mscLineNo 在 plistPmContMscTechDO 中查找匹配项
                Optional<PmContMscTechDO> matchingItem = plistPmContMscTechDO.stream()
                        .filter(item -> orderNo.equals(item.getOrderNo()) && mscLineNo.equals(item.getMscLineNo()))
                        .findFirst();
                if (!ultDetectMethodCode.isEmpty()) {
                    if(!coolCode.isEmpty()){
                        break;
                    }else{
                        coolCode = nvl(matchingItem.get().getCoolCode(),"");
                    }
                }else{
                    ultDetectMethodCode = nvl(matchingItem.get().getUltDetectMethodCode(),"");
                    if (coolCode.isEmpty()){
                        coolCode = nvl(matchingItem.get().getCoolCode(),"");
                    }
                }
            }
            pmPdSlabDO.setUltDetectMethodCode(ultDetectMethodCode);
            pmPdSlabDO.setCoolCode(coolCode);

            Optional<PmProContProdAttrDO> resultPmProContProdAttrDO = plistPmProContProdAttrDO.stream()
                    .filter(item -> ggcr.getLtActInfoGPreOrder().get(0).getOrderNo().equals(item.getOrderNo())) //TODO 根据轴合同号查询生产合同产品属性表  暂时取第一条合同
                    .findFirst();
            pmPdSlabDO.setWithSideFlag(resultPmProContProdAttrDO.get().getProdSideMark());

            pmPdSlabDO.setHeadCut(BigDecimal.valueOf(ggcr.getHead_Len()));
            pmPdSlabDO.setTailCut(BigDecimal.valueOf(ggcr.getTail_Len()));

            String urgOrderFlag = "";
            for (GPreOrder gPreOrder : ggcr.getLtActInfoGPreOrder()) {
                String orderNo = gPreOrder.getOrderNo();

                // 根据 orderNo 和 mscLineNo 在 plistPmContMscTechDO 中查找匹配项
                Optional<PmProContProdAttrDO> pmProContProdAttrDOItem = plistPmProContProdAttrDO.stream()
                        .filter(item -> orderNo.equals(item.getOrderNo()))
                        .findFirst();
                urgOrderFlag = String.valueOf(nvl(pmProContProdAttrDOItem.get().getUrgOrderFlag(),""));
                if(!urgOrderFlag.isEmpty()){
                    break;
                }
            }
            pmPdSlabDO.setUrgOrderFlag(urgOrderFlag);

            pmPdSlabDO.setSgSign(SInfo.getSgSign());
            pmPdSlabDO.setStNo(SInfo.getStNo());
            pmPdSlabDO.setOutSgNo(SInfo.getStNo());

            pmPdSlabDO.setPreCleanSlabThick(SInfo.getSlabActThk());
            pmPdSlabDO.setPreCleanSlabWidth(SInfo.getSlabActWid());
            pmPdSlabDO.setPreCleanSlabLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            pmPdSlabDO.setPreCleanSlabMaxLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            pmPdSlabDO.setPreCleanSlabMinLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            BigDecimal weight = SInfo.getSlabActThk().multiply(SInfo.getSlabActWid()).multiply(BigDecimal.valueOf(ggcr.getVirtualSlab_Len())).multiply(BigDecimal.valueOf(7.85));
            pmPdSlabDO.setPreCleanSlabWt(weight);
            pmPdSlabDO.setPreCleanSlabMaxWt(weight);
            pmPdSlabDO.setPreCleanSlabMinWt(weight);

            pmPdSlabDO.setInfurSlabThick(SInfo.getSlabActThk());
            pmPdSlabDO.setInfurSlabWid(SInfo.getSlabActWid());
            pmPdSlabDO.setInfurSlabLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            pmPdSlabDO.setInfurSlabMaxLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            pmPdSlabDO.setInfurSlabMinLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));
            pmPdSlabDO.setInfurSlabWt(weight);
            pmPdSlabDO.setInfurSlabMaxWt(weight);
            pmPdSlabDO.setInfurSlabMinWt(weight);

            pmPdSlabDO.setFixSlabLenFlag(0);
            pmPdSlabDO.setFixSlabLen(BigDecimal.valueOf(ggcr.getVirtualSlab_Len()));

            Optional<PmContMscTechDO> resultPmContMscTechDO1 = plistPmContMscTechDO.stream()
                    .filter(item -> ggcr.getLtActInfoGPreOrder().get(0).getOrderNo().equals(item.getOrderNo()) && ggcr.getLtActInfoGPreOrder().get(0).getInfoPmContMscTechDO().getMscLineNo().equals(item.getMscLineNo())) //TODO 根据轴合同号和产线号查询合同工艺表  暂时取第一条合同
                    .findFirst();

            pmPdSlabDO.setHpFlameCleanDiv(resultPmContMscTechDO1.get().getFlameCleanDiv());
            pmPdSlabDO.setSlabFinishCode(resultPmContMscTechDO1.get().getSlabFinishCode());

            Optional<PmPdSlabInfoDO> resultPmPdSlabInfoDO = plistPmPdSlabInfoDO.stream()
                    .filter(item -> ggcr.getSlabNo().equals(item.getSlabNo()))
                    .findFirst();
            pmPdSlabDO.setSlabPlaceCode(resultPmPdSlabInfoDO.get().getSlabPlaceCode());

            pmPdSlabDO.setRollAimThick(ggcr.getCal_Thk());
            pmPdSlabDO.setRollAimWidth(BigDecimal.valueOf(ggcr.getCal_Wid()));
            pmPdSlabDO.setRollAimLen(BigDecimal.valueOf(ggcr.getCal_Len()));
            pmPdSlabDO.setOrderThickAxis(ggcr.getLtActInfoGPreOrder().get(0).getThk());
            pmPdSlabDO.setCombinePlateWidth(BigDecimal.valueOf(ggcr.getLtActInfoGPreOrder().get(0).getWid()));
            pmPdSlabDO.setCombinePlateLen(BigDecimal.valueOf(ggcr.getLtActInfoGPreOrder().get(0).getLen()));
            pmPdSlabDO.setRollWtHp(weight);

            BigDecimal minThickTolMax = ggcr.getLtActInfoGPreOrder().stream()
                    .map(GPreOrder::getThickTolMax)
                    .min(BigDecimal::compareTo)
                    .orElse(BigDecimal.ZERO);
            pmPdSlabDO.setRollThickTolMax(minThickTolMax);
            BigDecimal maxThickTolMin = ggcr.getLtActInfoGPreOrder().stream()
                    .map(GPreOrder::getThickTolMin)
                    .max(BigDecimal::compareTo)
                    .orElse(BigDecimal.ZERO);
            pmPdSlabDO.setRollThickTolMin(maxThickTolMin);

            BigDecimal yield = (ggcr.getLtActInfoGPreOrder().stream()
                    .map(gPreOrder -> BigDecimal.valueOf(gPreOrder.getWid()).multiply(gPreOrder.getThk()).multiply(BigDecimal.valueOf(gPreOrder.getLen())))
                    .reduce(BigDecimal.ZERO, BigDecimal::add))
                    .multiply(BigDecimal.valueOf(7.85))
                    .divide(weight, 2, RoundingMode.HALF_UP);
            pmPdSlabDO.setCommandYield(yield);
            pmPdSlabDO.setNormYield(yield);
            pmPdSlabDO.setRollYield(yield);

            pmPdSlabDO.setEdgerUseFlag("Y");
            pmPdSlabDO.setBroadMasUseFlag("N");
            pmPdSlabDO.setFormMasUseFlag("N");
            pmPdSlabDO.setRollDirectCode("L"); //TODO 轧制方向代码 横轧C纵轧L  暂时默认L
            pmPdSlabDO.setSampleLenHead(ggcr.getSmp_Len());//TODO 头尾中 试样长度 暂时默认试样长度
            pmPdSlabDO.setSampleLenTail(ggcr.getSmp_Len());
            pmPdSlabDO.setSampleLenMid(ggcr.getSmp_Len());
            pmPdSlabDO.setRsCode("0");
            if(BigDecimal.valueOf(ggcr.getCal_Wid()).compareTo(SInfo.getSlabActWid()) > 0){
                pmPdSlabDO.setBroadFlag("1");  //需要展宽
            }else{
                pmPdSlabDO.setBroadFlag("0");
            }

            pmPdSlabDO.setOrderNoAxis(ggcr.getLtActInfoGPreOrder().get(0).getOrderNo());//TODO 轴合同号 暂时取第一个订单号
            pmPdSlabDO.setMscLineNo("1030"); //TODO 产线号，暂时默认1030

            pmPdSlabDO.setTransFlag(BigDecimal.valueOf(0));

            pmPdSlabDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmPdSlabDO.setLastUpdatedBy(781L);

            pmPdSlabDOList.add(pmPdSlabDO);
        }
    }

    //大板粗切位置表
    public void SetPmPdPlateCut(){
        for(int i = 0; i < gltGGrpCalRslt.size(); i++){
            GGrpCalRslt ggcr=gltGGrpCalRslt.get(i);
            PmPdPlateCutDO pmPdPlateCutDO = new PmPdPlateCutDO();

            pmPdPlateCutDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmPdPlateCutDO.setLastUpdatedBy(781L);
            pmPdPlateCutDOList.add(pmPdPlateCutDO);
        }
    }

    //组合子板位置表
    public void SetPmPdBlockPos(){
        for(int i = 0; i < gltGGrpCalRslt.size(); i++){
            PmPdBlockPosDO pmPdBlockPosDO = new PmPdBlockPosDO();
            pmPdBlockPosDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmPdBlockPosDO.setLastUpdatedBy(781L);
            pmPdBlockPosDOList.add(pmPdBlockPosDO);
        }
    }

    //组板合同总量表
    public void SetPmPdContSum(){
        for(int i = 0; i < gltGGrpCalRslt.size(); i++){
            PmPdContSumDO pmPdContSumDO = new PmPdContSumDO();
            pmPdContSumDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmPdContSumDO.setLastUpdatedBy(781L);
            pmPdContSumDOList.add(pmPdContSumDO);
        }
    }

    //组板小板位置表
    public void SetPmSmallPltPos(){
        for(int i = 0; i < gltGGrpCalRslt.size(); i++){
            PmSmallPltPosDO pmSmallPltPosDO = new PmSmallPltPosDO();
            pmSmallPltPosDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmSmallPltPosDO.setLastUpdatedBy(781L);
            pmSmallPltPosDOList.add(pmSmallPltPosDO);
        }
    }

    //命令板坯表
    public void SetPmCmdSlab(){
        for(int i = 0; i < gltGGrpCalRslt.size(); i++){
            PmCmdSlabDO pmCmdSlabDO = new PmCmdSlabDO();
            pmCmdSlabDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmCmdSlabDO.setLastUpdatedBy(781L);
            pmCmdSlabDOList.add(pmCmdSlabDO);
        }
    }

    //命令大板表
    public void SetPmCmdLargePlt(){
        for(int i = 0; i < gltGGrpCalRslt.size(); i++){
            PmCmdLargePltDO pmCmdLargePltDO = new PmCmdLargePltDO();
            pmCmdLargePltDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmCmdLargePltDO.setLastUpdatedBy(781L);
            pmCmdLargePltDOList.add(pmCmdLargePltDO);
        }
    }

    //命令参数表
    public void SetPmCmdCont(){
        for(int i = 0; i < gltGGrpCalRslt.size(); i++){
            PmCmdContDO pmCmdContDO = new PmCmdContDO();
            pmCmdContDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmCmdContDO.setLastUpdatedBy(781L);
            pmCmdContDOList.add(pmCmdContDO);
        }
    }

    //命令小板表
    public void SetPmCmdSmallPlt(){
        for(int i = 0; i < gltGGrpCalRslt.size(); i++){
            PmCmdSmallPltDO pmCmdSmallPltDO = new PmCmdSmallPltDO();
            pmCmdSmallPltDO.setCreatedBy(781L);//TODO 创建人修改人，暂时默认781  BKGY
            pmCmdSmallPltDO.setLastUpdatedBy(781L);
            pmCmdSmallPltDOList.add(pmCmdSmallPltDO);
        }
    }

}
